@@ -58,7 +58,6 @@ from schooltool.person.interfaces import IPerson
 from schooltool.gradebook.browser.gradebook import LinkedActivityGradesUpdater
 from schooltool.requirement.interfaces import IRangedValuesScoreSystem
 from schooltool.requirement.scoresystem import RangedValuesScoreSystem
-from schooltool.requirement.scoresystem import UNSCORED
 from schooltool.term.interfaces import ITerm, IDateManager
 
 
@@ -159,6 +158,8 @@ class UnhideWorksheetsView(object):
                 worksheet = removeSecurityProxy(self.context[name])
                 worksheet.hidden = False
             self.request.response.redirect(self.nextURL())
+        elif 'CANCEL' in self.request:
+            self.request.response.redirect(self.nextURL())
 
     def nextURL(self):
         return absoluteURL(self.context, self.request)
@@ -472,9 +473,11 @@ class WorksheetsExportView(export.ExcelExportView):
                      export.Text(student.first_name),
                      export.Text(student.last_name)]
             for activity in activities:
-                value, ss = gradebook.getEvaluation(student, activity)
-                if value is None or value is UNSCORED:
+                score = gradebook.getScore(student, activity)
+                if not score:
                     value = ''
+                else:
+                    value = score.value
                 cells.append(export.Text(value))
             for col, cell in enumerate(cells):
                 self.write(ws, starting_row+row, col, cell.data, **cell.style) 
